from logging import Logger
import logging
from backpack import Backpack
from state_manager import StateManager

class FluentLogger(Logger):
    def __init__(self, **kwargs):
        self.context = {}
        name = kwargs.pop("name", "logger")
        self.logger = logging.getLogger(name)
        self.backpack = {}
        self.backpack = kwargs.pop("backpack", Backpack())

    def with_item(self, key, value):
        self.backpack.with_item(key, value)
        return self

    def without_item(self, key):
        self.backpack.without_item(key)
        return self


    def with_one_time(self, key, value):
        self.backpack.with_one_time(key, value)
        return self
    
    def with_handler(self, handler):
        if self.backpack:
            handler.with_backpack(self.backpack)
        self.logger.addHandler(handler)
        return self

    def log_metric(self, message, name, value):
        self.backpack.with_metric(name, value)
        self.logger.warning(message)
        self.backpack.without_metric(name)
        
    def with_level(self, level):
        self.logger.setLevel(level)
        return self
        
    def info(self, msg):
        self.logger.info(msg)
        return self
        
    def error(self, msg):
        self.logger.error(msg)
        return self
        
    def with_timer(self, name, start_time=None):
        self.backpack.with_timer(name, start_time)
        return self
    
    def without_timer(self, name):
        self.backpack.without_timer(name)
        return self
        
    def persist(self, **kwargs):
        id = kwargs.pop('StateId', self.backpack.id)
        state_manager = StateManager()
        state_manager.upsert(self.backpack, StateId = id)
        
    def clone(self):
        logger = FluentLogger()
        for handler in self.logger.handlers:
            logger.with_handler(handler)
        for key in self.backpack.perm_items.keys():
            logger.with_item(key, self.backpack.perm_items[key])
        for timer in self.backpack.timers.keys():
            logger.with_timer(timer, self.backpack.timers[timer])
        logger.setLevel(self.logger.level)
        return logger
        


import boto3
import logging
import json
import datetime
import dateutil.parser

class KinesisFirehoseHandler(logging.Handler):
    """
    Amazon Kinesis Firehose logging handler
    """

    def __init__(self, *args, **kwargs):

        super(KinesisFirehoseHandler, self).__init__(
            level=kwargs.pop('level', logging.NOTSET))
        self.stream_name = kwargs.pop('stream_name', None)
        self.client = boto3.client('firehose')
        
    def with_backpack(self, backpack):
        self.backpack = backpack
        return self

    def emit(self, record):
        try:
            if self.backpack:
                record.__dict__.update(self.backpack.perm_items)
                record.__dict__.update(self.backpack.metrics)
                record.__dict__.update(self.backpack.one_time_items)
                if(self.backpack.timers):
                    for m in self.backpack.timers:
                        
                        delta = datetime.datetime.now() - dateutil.parser.parse(self.backpack.timers[m])
                        record.__dict__['{}InMs'.format(m)] = int(delta.total_seconds() * 1000)
            self.client.put_record(
                DeliveryStreamName=self.stream_name,
                Record={ 'Data': self.format(record) })
            if (self.backpack.one_time_items):
                self.backpack.one_time_items = {}
        except:
            raise
            self.handleError(record)




import uuid
import datetime
import json

class Backpack(object):
    def __init__(self, **kwargs):
        self.temp_items = {}
        self.metrics = {}
        self.perm_items = {}
        self.one_time_items = {}
        self.timers = {}
        self.perm_items['id'] = kwargs.pop("id", str(uuid.uuid4()))

    @property
    def id(self):
        return self.perm_items['id']
        
    def with_item(self, key, value):
        self.perm_items[key] = value
        return self
        
    def without_item(self, key):
        del self.perm_items[key]
        return self

    def with_one_time_item(self, key, value):
        self.one_time_items[key] = value
        return self

    def without_one_item_item(self, key):
        del self.one_time_items[key]
        return self
        
    def with_metric(self, key, value):
        metric = 'm_{}'.format(key)
        self.metrics[metric] = value
        return self
        
    def without_metric(self, key):
        metric = 'm_{}'.format(key)
        del self.metrics[metric]
        return self

    def with_timer(self, name, start_time=None):
        if not start_time:
            start_time = datetime.datetime.now().isoformat()
        self.timers[name] = start_time
        return self
        
    def without_timer(self, name):
        del self.timers[name]
        return self

    def to_json(self):
        b = {}
        b['id'] = self.perm_items['id']
        b['metrics'] = self.metrics
        b['perm_items'] = self.perm_items
        b['timers'] = self.timers
        return json.dumps(b)
        
    def from_json(self, j):
        backpack = json.loads(j)
        self.metrics = backpack['metrics']
        self.perm_items = backpack['perm_items']
        self.timers = backpack['timers']
        self.one_time_items = {}
        self.temp_items = {}
        return self


import json
import logging
import arrow

class SimpleJsonFormatter(logging.Formatter):
    """
    Simply JSON log formatter for Amazon Kinesis Firehose logging
    """

    def format(self, record):
        ret = {}
        for attr, value in record.__dict__.items():
            if attr == 'asctime':
                value = self.formatTime(record)
            if attr == 'exc_info' and value is not None:
                value = self.formatException(value)
            if attr == 'stack_info' and value is not None:
                value = self.formatStack(value)
            ret[attr] = value
        ret['timestamp'] = arrow.utcnow().isoformat()
        print(json.dumps(ret))
        return json.dumps(ret)


import boto3
import json
from botocore.exceptions import ClientError
from backpack import Backpack

class StateManager(object):
    def __init__(self, **kwargs):
        table_name = kwargs.pop('TableName', 'BackpackState')
        dynamodb = boto3.resource('dynamodb')
        self.table = dynamodb.Table(table_name)
    
    def upsert(self, backpack, **kwargs):
        state_id = kwargs.get('StateId', backpack.id)
        response = self.table.put_item(
            Item={
                'state_id': state_id,
                'perm_items': backpack.perm_items,
                'metrics': backpack.metrics,
                'timers': backpack.timers
            }
        )
        
    def get(self, state_id):
        try:
            response = self.table.get_item(
                Key={
                    'state_id': state_id
                }
            )
        except ClientError as e:
            print(e.response['Error']['Message'])
        else:
            b = Backpack().from_json(json.dumps(response['Item']))
            return b


import os
from setuptools import setup, find_packages

# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...

setup(
    name = "kitchen-sink-logger",
    version = "0.0.9",
    author = "Troy Larson",
    author_email = "troylar@gmail.com",
    description = ("Easily log everything, including the kitchen sink, if that's your thing"),
    license = "MIT",
    keywords = "logging",
    url = "http://packages.python.org/kitchen-sink-logger",
    packages=find_packages(),
    long_description='here',
    classifiers=[
        'License :: OSI Approved :: MIT License'
    ],
)
